﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

/*
 * UnityAssetMultiVamifier is a modification of NoStage3's UnityAssetVamifier
 * which allows per-material changes.
 * Color options aren't working correctly with this setup so I've commented them out.
 * If anyone wants to have a go at making them work, have at it! :D
 * -ClockwiseSilver
 *
 * Original author and license info below.
 * THANKS NoStage3!!!!
 *
 *----------------------------------------------------------------------------
 *
 * UnityAssetVamifier is a simple plugin to convert Materials used in
 * Unity Asset Bundles (.assetbundle) from Unity default shaders
 * (Roughness setup preferred) to the builtin VAM shader and expose
 * the shader properties in the UI.
 *
 * Authors: NoStage3
 * License: Creative Commons with Attribution (CC BY 3.0)
 * Current version: 1.10
 * 
 * Change for 1.10:
 * - message logging is now an option
 * - VaM 1.20 and higher - automatically applies to new asset when asset is changed
 * 
 * Changes for 1.9:
 * - bug fix for IBLFilter slider not setting initially
 *
 * Changes for 1.8:
 * - added sliders for diffuse and specular bumpiness
 *  
 * Changes for 1.7:
 * - fix: shaders with alphatest/alphablend keywords no longer get converted
 *
 * Changes for 1.6:
 * - material colors from unity are now converted better by default
 * - manual color inputs now only work when "Colorize" is toggled on
 *
 * Changes for 1.5:
 * - fixed bug where only first material of each renderer was converted
 *
 * Changes for 1.4:
 * - fixed loading bug for very large assets (that take long to load)
 * 
 * Changes for 1.3:
 * - tiling and offset values are now all taken from main texture
 * 
 * Changes for 1.2:
 * - Support for Texture tiling and offset
 * 
 * Changes for 1.1:
 * - Improved waiting for Asset Bundle solution
 * - Reordered UI to match VAM UI (sliders left, colors right)
 * 
 */
 

namespace MVRPlugin {
	public class MatPackage
	{
		public List<JSONStorableFloat> sliders = new List<JSONStorableFloat>();

		protected UIDynamicToggle allowColorizeButton;
		protected UIDynamicColorPicker diffColorPicker, specColorPicker, subdermisColorPicker;		
		protected JSONStorableFloat jSpecIntensityFloat, jSpecSharpnessFloat, jSpecFresnelFloat, jDiffOffsetFloat,
			jSpecOffsetFloat, jGlossOffsetFloat, jIBLFilterFloat, jDiffBumpinessFloat, jSpecBumpinessFloat;		
		protected JSONStorableBool jAllowColorizeBool;
		protected JSONStorableColor jDiffColor, jSpecColor, jSubdermisColor;		
		protected MVRPlugin.UnityAssetMultiVamifier script;
		
		public int listIndex = -1;
		public int rendererIndex;
		public int materialIndex;
		public Material material;
		public bool isConverted;
		public Color buttonColor = Color.blue;

		public MatPackage(int rIndex, int mIndex, int lIndex, Material mat, bool converted,
			MVRScript script, Color buttonColor) {
			this.rendererIndex = rIndex;
			this.materialIndex = mIndex;
			this.listIndex = lIndex;
			this.material = mat;
			this.isConverted = converted;
			this.script = script as MVRPlugin.UnityAssetMultiVamifier;
			this.buttonColor = buttonColor;
		}
				
		public void Init()
		{
			// Specular Intensity
			jSpecIntensityFloat = new JSONStorableFloat("SpecularIntensity" + listIndex.ToString(), 0.5f, SetSpecIntensity, 0f, 1f, true);
			sliders.Add(jSpecIntensityFloat);			

			// Specular Sharpness
			jSpecSharpnessFloat = new JSONStorableFloat("SpecularSharpness" + listIndex.ToString(), 6f, SetSpecSharpness, 0f, 10f, true);
			sliders.Add(jSpecSharpnessFloat);			

			// Specular Fresnel
			jSpecFresnelFloat = new JSONStorableFloat("SpecularFresnel" + listIndex.ToString(), 0f, SetSpecFresnel, 0f, 1f, true);
			sliders.Add(jSpecFresnelFloat);			

			// Diffuse Offset
			jDiffOffsetFloat = new JSONStorableFloat("DiffuseOffset" + listIndex.ToString(), 0f, SetDiffOffset, -1f, 1f, true);
			sliders.Add(jDiffOffsetFloat);			

			// Spec Offset
			jSpecOffsetFloat = new JSONStorableFloat("SpecularOffset" + listIndex.ToString(), 0f, SetSpecOffset, -1f, 1f, true);
			sliders.Add(jSpecOffsetFloat);			

			// Gloss Offset
			jGlossOffsetFloat = new JSONStorableFloat("GlossOffset" + listIndex.ToString(), 0.8f, SetGlossOffset, 0, 1f, true);
			sliders.Add(jGlossOffsetFloat);			

			// IBL Filter (affects Global Illum Skybox ?!)
			jIBLFilterFloat = new JSONStorableFloat("IBLFilter" + listIndex.ToString(), 0f, SetIBLFilter, 0, 1f, true);
			sliders.Add(jIBLFilterFloat);			

			// Diffuse Bumpiness
			jDiffBumpinessFloat = new JSONStorableFloat("DiffuseBumpiness" + listIndex.ToString(), 1f, SetDiffBumpiness, 0, 3f, true);
			sliders.Add(jDiffBumpinessFloat);			

			// Spec Bumpiness
			jSpecBumpinessFloat = new JSONStorableFloat("SpecularBumpiness" + listIndex.ToString(), 1f, SetSpecBumpiness, 0, 3f, true);
			sliders.Add(jSpecBumpinessFloat);
			
			foreach(JSONStorableFloat slider in sliders){ script.RegisterFloat(slider); }
			
			/*
			//allowColorizeButton
			jAllowColorizeBool = new JSONStorableBool("Colorize" + listIndex.ToString(), false);			
			script.RegisterBool(jAllowColorizeBool);
			
			//Diff Color
			HSVColor diffColorHSVC = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
			jDiffColor = new JSONStorableColor("DiffuseColor" + listIndex.ToString(), diffColorHSVC, SetDiffColor);
			script.RegisterColor(jDiffColor);
			
			//Specular Color
			HSVColor specColorHSVC = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
			jSpecColor = new JSONStorableColor("SpecularColor" + listIndex.ToString(), specColorHSVC, SetSpecColor);
			script.RegisterColor(jSpecColor);
			
			//Subdermis Color
			HSVColor subdermisColorHSVC = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
			jSubdermisColor = new JSONStorableColor("SubdermisColor" + listIndex.ToString(), subdermisColorHSVC, SetSubdermisColor);
			script.RegisterColor(jSubdermisColor);
			*/
		}
		
		public void CreateMenu()
		{
			foreach(JSONStorableFloat slider in sliders)
			{
				script.CreateSlider(slider, true);				
			}
			
			/*
			allowColorizeButton = script.CreateToggle(jAllowColorizeBool, true);
			if (jAllowColorizeBool.val) { BuildColorUI(); ApplyJStoredColors(); }
			allowColorizeButton.toggle.onValueChanged.AddListener(delegate (bool state) {
				if (state) {
					BuildColorUI();
					ApplyJStoredColors();
				} else {
					RemoveColorUI();
					script.RestoreOriginalColors(this);
				}
			});
			*/
		}
		
		public void LoadAll()
		{
			/*
			JSONNode node;
			if (GetStoredNode(jAllowColorizeBool.name, out node))
			{
				jAllowColorizeBool.valNoCallback = node.AsBool;
			}
			
			if (GetStoredNode(jDiffColor.name, out node))
			{
				HSVColor color;
				color.H = node["h"].AsFloat;
				color.S = node["s"].AsFloat;
				color.V = node["v"].AsFloat;
				jDiffColor.valNoCallback = color;
			}
			
			if (GetStoredNode(jSpecColor.name, out node))
			{
				HSVColor color;
				color.H = node["h"].AsFloat;
				color.S = node["s"].AsFloat;
				color.V = node["v"].AsFloat;
				jSpecColor.valNoCallback = color;
			}
			
			if (GetStoredNode(jSubdermisColor.name, out node))
			{
				HSVColor color;
				color.H = node["h"].AsFloat;
				color.S = node["s"].AsFloat;
				color.V = node["v"].AsFloat;
				jSubdermisColor.valNoCallback = color;
			}
			*/
			
			JSONNode node;
			foreach(JSONStorableFloat slider in sliders)
			{
				if (GetStoredNode(slider.name, out node))
				{
					slider.valNoCallback = node.AsFloat;
				}
			}
			
			ApplyStored();
		}
		
		public bool GetStoredNode(string storableName, out JSONNode node)
		{
			foreach (JSONNode a in SuperController.singleton.loadJson["atoms"].AsArray)
			{
				if (a["id"].Value == script.containingAtom.name)
				{
					foreach (JSONNode n in a["storables"].AsArray)
					{
						if (n["id"].Value == script.storeId)
						{
							if (n[storableName].Value != "")
							{
								node = n[storableName];
								return true;
							}
						}
					}
				}
			}	
			node = null;
			return false;
		}
		
		public void ClearMenu()
		{
			foreach(JSONStorableFloat slider in sliders) { script.RemoveSlider(slider); }
			//if (jAllowColorizeBool.val) { RemoveColorUI(); }
			//script.RemoveToggle(allowColorizeButton);
		}
		
		public void ApplyJStoredColors() {
			material.SetColor("_Color", HSVColorPicker.HSVToRGB(jDiffColor.val));
			material.SetColor("_SpecColor", HSVColorPicker.HSVToRGB(jSpecColor.val));
			material.SetColor("_SubdermisColor", HSVColorPicker.HSVToRGB(jSubdermisColor.val));
		}
		
		public void ApplyStored()
		{
			SetSpecIntensity(jSpecIntensityFloat);
			SetSpecSharpness(jSpecSharpnessFloat);
			SetSpecFresnel(jSpecFresnelFloat);
			SetDiffOffset(jDiffOffsetFloat);
			SetSpecOffset(jSpecOffsetFloat);
			SetGlossOffset(jGlossOffsetFloat);
			SetIBLFilter(jIBLFilterFloat);
			SetDiffBumpiness(jDiffBumpinessFloat);
			SetSpecBumpiness(jSpecBumpinessFloat);
			//ApplyJStoredColors();
		}

		protected void SetSpecIntensity(JSONStorableFloat jf) {
			if (!isConverted) return;
			material.SetFloat("_SpecInt", jf.val);
		}

		protected void SetSpecSharpness(JSONStorableFloat jf) {
			if (!isConverted) return;
			material.SetFloat("_Shininess", jf.val);
		}
		
		protected void SetSpecFresnel(JSONStorableFloat jf) {
			if (!isConverted) return;
			material.SetFloat("_Fresnel", jf.val);
		}

		protected void SetDiffOffset(JSONStorableFloat jf) {
			if (!isConverted) return;
			material.SetFloat("_DiffOffset", jf.val);
		}

		protected void SetSpecOffset(JSONStorableFloat jf) {
			if (!isConverted) return;
			material.SetFloat("_SpecOffset", jf.val);
		}

		protected void SetGlossOffset(JSONStorableFloat jf) {
			if (!isConverted) return;
			material.SetFloat("_GlossOffset", jf.val);
		}
		
		protected void SetIBLFilter(JSONStorableFloat jf) {
			if (!isConverted) return;
			material.SetFloat("_IBLFilter", jf.val);
		}

		protected void SetDiffBumpiness(JSONStorableFloat jf) {
			if (!isConverted) return;
			material.SetFloat("_DiffuseBumpiness", jf.val);
		}
		
		protected void SetSpecBumpiness(JSONStorableFloat jf) {
			if (!isConverted) return;
			material.SetFloat("_SpecularBumpiness", jf.val);
		}

		protected void SetDiffColor(JSONStorableColor jcolor) {
			if (!jAllowColorizeBool.val || !isConverted) return;
			material.SetColor("_Color", jcolor.colorPicker.currentColor);
		}

		protected void SetSpecColor(JSONStorableColor jcolor) {
			if (!jAllowColorizeBool.val || !isConverted) return;
			material.SetColor("_SpecColor", jcolor.colorPicker.currentColor);
		}

		protected void SetSubdermisColor(JSONStorableColor jcolor) {
			if (!jAllowColorizeBool.val || !isConverted) return;
			material.SetColor("_SubdermisColor", jcolor.colorPicker.currentColor);
		}
		
		protected void BuildColorUI() {
			diffColorPicker = script.CreateColorPicker(jDiffColor, true);
			specColorPicker = script.CreateColorPicker(jSpecColor, true);
			subdermisColorPicker = script.CreateColorPicker(jSubdermisColor, true);
		}

		protected void RemoveColorUI() {
			script.RemoveColorPicker(diffColorPicker);
			script.RemoveColorPicker(specColorPicker);
			script.RemoveColorPicker(subdermisColorPicker);
		}
	}
	
	public class UnityAssetMultiVamifier : MVRScript {
		private readonly string VERSION_NUMBER = "1.10";

		private readonly List<string> UNITY_SHADER_NAMES = new List<string>(new string[] {
			"Standard",
			"Standard (Specular setup)",
			"Standard (Roughness setup)"
		});
		private readonly List<string> INVALID_SHADER_KEYWORDS = new List<string>(new string[] {
			"_ALPHAPREMULTIPLY_ON",
			"_ALPHATEST_ON",
			"_ALPHABLEND_ON"
		});
		private readonly string VAM_SHADER_NAME = "Custom/Subsurface/GlossNMCull";

		protected UIDynamicToggle showMessagesToggle;		

		protected JSONStorableBool showMessagesBool;

		protected List<MatPackage> origMaterials = new List<MatPackage>();
		protected List<MatPackage> newMaterials = new List<MatPackage>();
		protected Shader vamPropShader;
		protected List<Renderer> allRenderers = new List<Renderer>();
		protected CustomUnityAssetLoader customUnityAssetLoader;
		
		protected List<UIDynamicButton> buttons = new List<UIDynamicButton>();
		protected MatPackage currentMatPackage;

		public override void Init() {
			try {

				// create new VAM material from built-in shader
				vamPropShader = Shader.Find(VAM_SHADER_NAME);

				// build/load/populate material controls for VAM UI
				BuildUIControls();
				AddVersionNumberToUI();

				// get the loader component
				customUnityAssetLoader = containingAtom.GetComponentInChildren<CustomUnityAssetLoader>();
#if VAM_GT_1_20
				if (customUnityAssetLoader != null) {
					customUnityAssetLoader.RegisterAssetClearedCallback(AssetCleared);
					customUnityAssetLoader.RegisterAssetLoadedCallback(AssetLoaded);
					if (customUnityAssetLoader.isAssetLoaded) {
						// already loaded - sync up
						AssetLoaded();
					}
				}
#else
				// since there is no "asset bundle loaded" callback yet, we have to wait
				StartCoroutine(WaitForAssetBundle());
#endif

			}
			catch (Exception e) {
				SuperController.LogError("UnityAssetVamifier: Exception caught: " + e);
			}
		}
		
		public void MatButtonClicked(int i)
		{
			if (currentMatPackage != null) { currentMatPackage.ClearMenu(); }
			if (i >= newMaterials.Count) { SuperController.LogError("Invalid Material Index " + i.ToString()); return; }
			for (int j = 0; j < buttons.Count; j++)
			{
				if (newMaterials[j] != null && buttons[j] != null)
				{
					buttons[j].buttonColor = newMaterials[j].buttonColor;
				    buttons[j].textColor = Color.white;
				}
			}
			currentMatPackage = newMaterials[i];
			if (currentMatPackage != null) { currentMatPackage.CreateMenu(); }
			if (buttons[i] != null) { buttons[i].buttonColor = Color.yellow; buttons[i].textColor = Color.black; }
		}

		protected void BuildUIControls() {

			// showMessagesToggle
			showMessagesBool = new JSONStorableBool("Show Messages", true);
			RegisterBool(showMessagesBool);
			showMessagesToggle = CreateToggle(showMessagesBool, true);
		}

		protected void AddVersionNumberToUI() {
			JSONStorableString versionString = new JSONStorableString("pluginVersion",
			"UnityAssetMultiVamifier, a mod by\nClockwiseSilver based on\nUnityAssetVamifier "+ VERSION_NUMBER + " by NoStage3");
			UIDynamicTextField textFieldVersion = CreateTextField(versionString, true);
			textFieldVersion.height = 130;
		}


		// determine if the material should be converted or not
		protected bool IsValidMaterial(Material mat) {
			if (!UNITY_SHADER_NAMES.Contains(mat.shader.name))
				return false;

			foreach (string keyword in INVALID_SHADER_KEYWORDS) {
				if (mat.IsKeywordEnabled(keyword))
					return false;
			}
			return true;
		}

		protected void AssetCleared() {
			// don't need to reset materials because the asset would have been destroyed
			allRenderers.Clear();
			origMaterials.Clear();
			newMaterials.Clear();
		}

		protected void AssetLoaded() {
			containingAtom.GetComponentsInChildren<Renderer>(false, allRenderers);
			ConvertMaterial();
		}

		// wait for the asset bundle to be loaded by checking for renderers every 0.5s (a bit hacky...)
		protected IEnumerator WaitForAssetBundle() {

			while (allRenderers.Count < 1) {
				containingAtom.GetComponentsInChildren<Renderer>(false, allRenderers);
				yield return new WaitForSeconds(0.5f);
			}

			ConvertMaterial();
		}

		protected void ConvertMaterial() {
			List<Material> invalidMaterials = new List<Material>();

			if (allRenderers.Count == 0) {
				if (showMessagesBool.val) {
					SuperController.LogMessage("UnityAssetVamifier: Make sure you have an .assetbundle loaded and a prop selected from the Asset dropdown. Then reload this plugin.");
				}
#if VAM_GT_1_20
				else {
					SuperController.LogMessage("UnityAssetVamifier: Make sure you have an .assetbundle loaded and a prop selected from the Asset dropdown. Then reload this plugin.", true, false);
				}
#endif
				return;
			}

			// fill newMaterials with either the original material or an new material with the vam shader
			foreach (UIDynamicButton button in buttons) { RemoveButton(button); }
			buttons.Clear();
			for (int i = 0; i < allRenderers.Count; i++) {
				Color buttonColor = new Color(
					UnityEngine.Random.Range(0f, 0.7f),UnityEngine.Random.Range(0f, 0.7f),UnityEngine.Random.Range(0f, 0.7f));
				for (int j = 0; j < allRenderers[i].materials.Length; j++) {
					origMaterials.Add(new MatPackage(i, j, -1, allRenderers[i].materials[j], false, this, buttonColor));

					if (IsValidMaterial(allRenderers[i].materials[j]))
					{
						int idx = buttons.Count;
						MatPackage matPackage = new MatPackage(i, j, idx, new Material(vamPropShader), true, this, buttonColor);
						matPackage.Init();
						newMaterials.Add(matPackage);
						UIDynamicButton btn = CreateButton(
							allRenderers[i].gameObject.name.Replace("(Clone)", " ") +
							allRenderers[i].materials[j].name.Replace("(Instance)", ""),
							false);
						btn.buttonColor = buttonColor;
						btn.textColor = Color.white;						
						buttons.Add(btn);
						btn.button.onClick.AddListener(() => { MatButtonClicked(idx); });
					}
					else
					{
						newMaterials.Add(new MatPackage(i, j, -1, allRenderers[i].materials[j], false, this, buttonColor));
						invalidMaterials.Add(allRenderers[i].materials[j]);
					}
				}
			}
			
			// output invalid/skipped materials
			foreach (Material invalidMaterial in invalidMaterials) {
				if (showMessagesBool.val) {
					SuperController.LogMessage("UnityAssetVamifier: Material skipped: " + invalidMaterial.name + " -- Shader: [" + invalidMaterial.shader.name + "]");
				}
#if VAM_GT_1_20
				else {
					SuperController.LogMessage("UnityAssetVamifier: Material skipped: " + invalidMaterial.name + " -- Shader: [" + invalidMaterial.shader.name + "]", true, false);
				}
#endif
			}


			if (newMaterials.Count == 0) {
				SuperController.LogError("UnityAssetVamifier: No compatible material could be found on this asset. Check message log for skipped materials/shaders.");
				return;
			}


			//collect materials (matsToAssign) to assign to each renderer (needs to be assigned as array...)
			for (int i = 0; i < allRenderers.Count; i++)
			{
				Material[] matsToAssign = allRenderers[i].materials;
				for (int j = 0; j < allRenderers[i].materials.Length; j++)
				{
					if (IsValidMaterial(allRenderers[i].materials[j]))
					{
						Material origMat = GetOrigMaterialByIndex(i, j);
						Material vamMat = GetVamMaterialByIndex(i, j);
						Vector2 mainTexScale = origMat.GetTextureScale("_MainTex");
						Vector2 mainTexOffset = origMat.GetTextureOffset("_MainTex");

						// reassign colors (can get overwritten by colorize toggle)
						if (origMat.HasProperty("_Color")) {
							vamMat.SetColor("_Color", origMat.GetColor("_Color"));
						}
						if (origMat.HasProperty("_SpecColor")) {
							vamMat.SetColor("_SpecColor", origMat.GetColor("_SpecColor"));
						}
						if (origMat.HasProperty("_SubdermisColor")) {
							vamMat.SetColor("_SubdermisColor", origMat.GetColor("_SubdermisColor"));
						}

						// reassign textures and tiling from unity to vam material
						// use tiling from main texture since supported unity shaders don't allow tiling per texture type

						if (origMat.HasProperty("_MainTex")) {
							vamMat.SetTexture("_MainTex", origMat.GetTexture("_MainTex"));
							vamMat.SetTextureScale("_MainTex", mainTexScale);
							vamMat.SetTextureOffset("_MainTex", mainTexOffset);
						}

						if (origMat.HasProperty("_BumpMap")) {
							vamMat.SetTexture("_BumpMap", origMat.GetTexture("_BumpMap"));
							vamMat.SetTextureScale("_BumpMap", mainTexScale);
							vamMat.SetTextureOffset("_BumpMap", mainTexOffset);
						}

						if (origMat.HasProperty("_SpecGlossMap")) {
							vamMat.SetTexture("_SpecTex", origMat.GetTexture("_SpecGlossMap"));
							vamMat.SetTextureScale("_SpecTex", mainTexScale);
							vamMat.SetTextureOffset("_SpecTex", mainTexOffset);
						}

						if (origMat.HasProperty("_MetallicGlossMap")) {
							vamMat.SetTexture("_GlossTex", origMat.GetTexture("_MetallicGlossMap"));
							vamMat.SetTextureScale("_GlossTex", mainTexScale);
							vamMat.SetTextureOffset("_GlossTex", mainTexOffset);
						}

						matsToAssign[j] = vamMat;
					}
				}
				// assign VAM materials to prop renderer as array
				allRenderers[i].materials = matsToAssign;
			}

			foreach (MatPackage matPackage in newMaterials)
			{
				matPackage.LoadAll();
			}
			if (buttons.Count >= 0) MatButtonClicked(0);			
		}

		protected Material GetOrigMaterialByIndex(int rIndex, int mIndex) {
			foreach (MatPackage matPackage in origMaterials) {
				if (matPackage.rendererIndex == rIndex && matPackage.materialIndex == mIndex) {
					return matPackage.material;
				}
			}
			return null;
		}

		protected Material GetVamMaterialByIndex(int rIndex, int mIndex) {
			foreach (MatPackage matPackage in newMaterials) {
				if (matPackage.rendererIndex == rIndex && matPackage.materialIndex == mIndex) {
					return matPackage.material;
				}
			}
			return null;
		}


		void OnDestroy() {
			RestoreOriginalMaterials();
#if VAM_GT_1_20
				if (customUnityAssetLoader != null) {
					customUnityAssetLoader.DeregisterAssetClearedCallback(AssetCleared);
					customUnityAssetLoader.DeregisterAssetLoadedCallback(AssetLoaded);
				}
#endif
		}

		protected void RestoreOriginalMaterials() {
			for (int i = 0; i < allRenderers.Count; i++) {
				if (allRenderers[i] != null) {   // renderer may have been destroyed already by atom remove
					List<Material> matsToAssign = new List<Material>();
					for (int j = 0; j < allRenderers[i].materials.Length; j++) {
						matsToAssign.Add(GetOrigMaterialByIndex(i, j));
					}
					allRenderers[i].materials = matsToAssign.ToArray();
				}
			}
		}

		public void RestoreOriginalColors(MatPackage matPackage) {
				if (GetOrigMaterialByIndex(matPackage.rendererIndex, matPackage.materialIndex).HasProperty("_Color")) {
					matPackage.material.SetColor("_Color", GetOrigMaterialByIndex(matPackage.rendererIndex, matPackage.materialIndex).GetColor("_Color"));
				} else {
					matPackage.material.SetColor("_Color", Color.white);
				}
				if (GetOrigMaterialByIndex(matPackage.rendererIndex, matPackage.materialIndex).HasProperty("_SpecColor")) {
					matPackage.material.SetColor("_SpecColor", GetOrigMaterialByIndex(matPackage.rendererIndex, matPackage.materialIndex).GetColor("_SpecColor"));
				} else {
					matPackage.material.SetColor("_SpecColor", Color.white);
				}
				if (GetOrigMaterialByIndex(matPackage.rendererIndex, matPackage.materialIndex).HasProperty("_SubdermisColor")) {
					matPackage.material.SetColor("_SubdermisColor", GetOrigMaterialByIndex(matPackage.rendererIndex, matPackage.materialIndex).GetColor("_SubdermisColor"));
				} else {
					matPackage.material.SetColor("_SubdermisColor", Color.white);
				}
		}
	}
}